#include	"RingBuff.h"

//--------------------------------------------------------
// Function
//	クラスの構築、バッファの最大サイズを指定
// Parameter
//	long	lMaxSize	[in]格納可能な最大サイズ	
// Return
//	なし
//--------------------------------------------------------
CRingBuff::CRingBuff(long lMaxSize)
{
	m_fInitialized = FALSE;
	m_pbData = NULL;
	m_lMaxSize = lMaxSize;					// サイズを覚える
	m_lAllocSize = m_lMaxSize + 1;			// メモリを確保するサイズ
	m_lHead = 0;							// 読み込み開始位置
	m_lTail = 0;							// 書き込み開始位置
	m_pCMySyncObject = new CMySyncObject();
	m_pCMySyncObject->Initialize();

	if (lMaxSize > 0)
		m_pbData = (LPBYTE)calloc(m_lAllocSize, sizeof(BYTE));
	if (m_pbData != NULL)
		m_fInitialized = TRUE;
}

//--------------------------------------------------------
// Function
//	クラスの破棄、バッファの開放
// Parameter
//	なし
// Return
//	なし
//--------------------------------------------------------
CRingBuff::~CRingBuff(void)
{
	SAFE_FREE(m_pbData)
	m_pCMySyncObject->Uninitialize();
	SAFE_DELETE(m_pCMySyncObject)
	m_fInitialized = FALSE;
}

//-------------------------------------------------------------------------
// Function
//		データの読み込み
//		現在の読み込み開始位置から読み込む
// Parameter
//		LPBYTE	pbBuf	[in/out]バッファ
//		long	lSize	[in]バッファサイズ
// Return
//		実際に読み込んだサイズ <= バッファサイズ
//-------------------------------------------------------------------------
long CRingBuff::Read(LPBYTE pbBuf, long lSize)
{
	long	lReadSize = 0;
	long	ii;

// 読み込み可能サイズ(格納されているデータ数)
// 以上は読み込まないようにする
	m_pCMySyncObject->Lock();
	lReadSize = min(GetReadableSize0(), lSize);

	for (ii = 0; ii < lReadSize; ++ii)
	{
		pbBuf[ii] = m_pbData[m_lHead++];	// 読み込み位置更新
		m_lHead = m_lHead % m_lAllocSize;
	}
	m_pCMySyncObject->UnLock();
	return(lReadSize);
}

//-------------------------------------------------------------------------
// Function
//		データの仮読み込み
//		現在の読み込み開始位置から読み込む
//		読み込み位置の更新をしない
// Parameter
//		LPBYTE	pbBuf		[in/out]バッファ
//		long	lSize		[in]バッファサイズ
// Return
//		実際に読み込んだサイズ
//-------------------------------------------------------------------------
long CRingBuff::ReadWithoutUpdateHeadPoint(LPBYTE pbBuf, long lSize)
{
	long	lReadSize = 0;
	long	lHead, ii;

	// 読み込み可能サイズ(格納されているデータ数)
	// 以上は読み込まないようにする
	m_pCMySyncObject->Lock();
	lReadSize = min(GetReadableSize0(), lSize);

	lHead = m_lHead;
	for (ii = 0; ii < lReadSize; ++ii)
	{
		pbBuf[ii] = m_pbData[lHead++];	// 読み込み位置は更新していない
		lHead = lHead % m_lAllocSize;
	}
	m_pCMySyncObject->UnLock();
	return(lReadSize);
}

//-------------------------------------------------------------------------
// Function
//		読み込み位置の更新をする(指定バイト読んだことにする)
// Parameter
//		long	lSize	[in]読んだことにするバイト数
// Return
//		なし
//-------------------------------------------------------------------------
void CRingBuff::UpdateHeadPoint(long lSize)
{
	m_pCMySyncObject->Lock();
	m_lHead = (m_lHead + lSize) % m_lAllocSize;
	m_pCMySyncObject->UnLock();
}

//-------------------------------------------------------------------------
// Function
//		データの書き込み
//		現在の書き込み開始位置から書き込む
//		この関数を呼ぶ前にGetWriteableSizeで書き込み可能サイズを
//		調べること
// Parameter
//		LPBYTE	pbBuf		[in]バッファ
//		long	lSize		[in]バッファサイズ
// Return
//		TRUE:書き込み成功 FALSE:空き不足
//-------------------------------------------------------------------------
BOOL CRingBuff::Write(LPBYTE pbBuf, long lSize)
{
	BOOL    fRet = FALSE;
	long    ii;

	// 書き込み可能サイズ(バッファの空き)を取得し書き込めるか調べる
	m_pCMySyncObject->Lock();
	if (lSize > GetWriteableSize0())
		goto L_END;

	for (ii = 0; ii < lSize; ++ii)
	{
		m_pbData[m_lTail++] = pbBuf[ii];
		m_lTail = m_lTail % m_lAllocSize;
	}
	fRet = TRUE;
L_END:
	m_pCMySyncObject->UnLock();
	return(fRet);
}

//-------------------------------------------------------------------------
// Function
//		読み込み可能サイズ(格納されているデータ数)の取得
// Parameter
//		なし
// Return
//		読み込み可能サイズ
//-------------------------------------------------------------------------
long CRingBuff::GetReadableSize()
{
	long	lHead, lTail, lReadable = 0;

	m_pCMySyncObject->Lock();
	if (m_pbData == NULL)
		goto L_END;

	lHead = m_lHead;
	lTail = m_lTail;
	lReadable = (lHead <= lTail) ? (lTail - lHead) : (m_lAllocSize - lHead) + lTail;

L_END:
	m_pCMySyncObject->UnLock();
	return(lReadable);
}

//-------------------------------------------------------------------------
// Function
//		読み込み可能サイズ(格納されているデータ数)の取得
//		同期なし版
// Parameter
//		なし
// Return
//		読み込み可能サイズ
//-------------------------------------------------------------------------
long CRingBuff::GetReadableSize0()
{
	long	lHead, lTail, lReadable = 0;

	if (m_pbData == NULL)
		goto L_END;

	lHead = m_lHead;
	lTail = m_lTail;
	lReadable = (lHead <= lTail) ? (lTail - lHead) : (m_lAllocSize - lHead) + lTail;

L_END:
	return(lReadable);
}

//-------------------------------------------------------------------------
// Function
//		書き込み可能サイズ(空きサイズ)の取得
// Parameter
//		なし
// Return
//		書き込み可能サイズ
//-------------------------------------------------------------------------
long CRingBuff::GetWriteableSize()
{
	long	lHead, lTail, lWritable = 0;

	m_pCMySyncObject->Lock();
	if (m_pbData == NULL)
		goto L_END;

	lTail = m_lTail;
	lHead = m_lHead;
	lWritable = (lTail < lHead) ? (lHead - lTail) : (m_lAllocSize - lTail) + lHead;

	if (lWritable > 0)			// HaedとTailの間に隙間を開けるため1バイト減らす
		--lWritable;
L_END:
	m_pCMySyncObject->UnLock();
	return(lWritable);
}

//-------------------------------------------------------------------------
// Function
//		書き込み可能サイズ(空きサイズ)の取得
//		同期なし版
// Parameter
//		なし
// Return
//		書き込み可能サイズ
//-------------------------------------------------------------------------
long CRingBuff::GetWriteableSize0()
{
	long	lHead, lTail, lWritable = 0;

	if (m_pbData == NULL)
		goto L_END;

	lTail = m_lTail;
	lHead = m_lHead;
	lWritable = (lTail < lHead) ? (lHead - lTail) : (m_lAllocSize - lTail) + lHead;

	if (lWritable > 0)			// HaedとTailの間に隙間を開けるため1バイト減らす
		--lWritable;
L_END:
	return(lWritable);
}

//-------------------------------------------------------------------------
// Function
//		書き込み、読み込みポインタの初期化
// Parameter
//		なし
// Return
//		なし
//-------------------------------------------------------------------------
void CRingBuff::Clear()
{
	m_pCMySyncObject->Lock();
	m_lTail = 0;
	m_lHead = 0;
	m_pCMySyncObject->UnLock();
}

